#!/usr/bin/python

import string
import re
import sys

def spaces(l):
  ret = ''

  if l <= 0:
    return ret

  for i in range(l):
    ret += ' '

  return ret
# end spaces()

def pad_num(n):
  if n <= 0:
    return '0000'
  elif n < 10:
  	return '000' + str(n)
  elif n < 100:
  	return '00' + str(n)
  elif n < 1000:
  	return '0' + str(n)

  return n
# end pad_num()

def write_copy_file(source, dest_file):
  tf = open('./pydata/' + source + '.txt', 'r')

  for line in tf.readlines():
    dest_file.write(line)

  dest_file.write('\n')
  tf.close()
# end write_copy_file()

p_file = open('../StellarObjectTypes.txt', 'w')
x_file = open('../XFileClasses_Stellar.txt', 'w')

p_sizes = ['Tiny', 'Small', 'Medium', 'Large', 'Huge']

p_model_size = {'Tiny':    '8.0',
                'Small':  '10.0',
                'Medium': '12.0',
                'Large':  '14.0',
                'Huge':   '16.0'}

p_atm = ['Carbon Dioxide', 'Hydrogen', 'Methane', 'None', 'Oxygen']

x_atm_glow = {'Carbon Dioxide': '0, 164, 0',
              'Hydrogen':       '255, 64, 0',
              'Methane':        '164, 0, 255',
              'Oxygen':         '0, 64, 255',
              'None':           '0, 0, 0'}

p_icons = {'Carbon Dioxide': {'Gas': 11, 'Ice': 161, 'Rock': 446},
           'Hydrogen':       {'Gas': 1, 'Ice': 146, 'Rock': 506},
           'Methane':        {'Gas': 196, 'Ice': 116, 'Rock': 476},
           'Oxygen':         {'Gas': 16, 'Ice': 176, 'Rock': 386},
           'None':           {'Gas': 0, 'Ice': 216, 'Rock': 536}}

p_icon_offset = {'Tiny':   4,
                 'Small':  3,
                 'Medium': 2,
                 'Large':  1,
                 'Huge':   0}

p_port  = {'Carbon Dioxide': {'Gas': '0003', 'Ice': '0011', 'Moon': '0010', 'Terr': '0015', 'Ven': '0007'},
           'Hydrogen':       {'Gas': '0001', 'Ice': '0013', 'Moon': '0006', 'Terr': '0001', 'Ven': '0019'},
           'Methane':        {'Gas': '0002', 'Ice': '0005', 'Moon': '0016', 'Terr': '0005', 'Ven': '0010'},
           'Oxygen':         {'Gas': '0004', 'Ice': '0007', 'Moon': '0018', 'Terr': '0003', 'Ven': '0001'},
           'None':           {'Gas': '0000', 'Ice': '0023', 'Moon': '0005', 'Terr': '0010', 'Ven': '0025'}}

p_types = ['Gas', 'Ice', 'Rock']

p_classes = {'Gas':  ['Gas'],
             'Ice':  ['Ice'],
             'Rock': ['Moon', 'Terr', 'Ven']}

def write_planet(out_file, pdata, pnum):
  out_file.write('Stellar Object Name        := Planet ' + pdata['class'] + ' ' + pdata['atm'].split()[0] + ' ' + pdata['size'] + ' ' + str(pnum) + '\n')

  out_file.write('Physical Type              := Planet\n')
  out_file.write('XFile Class Name           := Planet ' + pdata['class'] + ' ' + pdata['atm'].split()[0] + ' ' + pdata['size'] + ' ' + str(pnum) + '\n')
  out_file.write('Maximum Model Size         := ' + p_model_size[pdata['size']] + '\n')
  out_file.write('Hex Size                   := 1\n')

  out_file.write('Picture Index              := ' + str(p_icons[pdata['atm']][pdata['type']] + p_icon_offset[pdata['size']]) + '\n')

  out_file.write('Portrait Filename          := planet_' + pdata['class'].lower() + '_' + p_port[pdata['atm']][pdata['class']] + '_' + pdata['size'].lower() + '.bmp\n')
  out_file.write('Large Portrait Filename    := planet_' + pdata['class'].lower() + '_' + p_port[pdata['atm']][pdata['class']] + '_' + pdata['size'].lower() + '_lp.jpg\n')
  out_file.write('Description                := \n')
  out_file.write('Planet Size                := ' + pdata['size'] + '\n')

  if pdata['type'] == 'Gas':
    out_file.write('Planet Physical Type       := ' + pdata['type'] + ' Giant\n')
  else:
    out_file.write('Planet Physical Type       := ' + pdata['type'] + '\n')

  out_file.write('Planet Atmosphere          := ' + pdata['atm'] + '\n')

  if pdata['type'] == 'Gas':
    out_file.write('Planet Gravity             := Formula Planet Gas Gravity\n')
  else:
    out_file.write('Planet Gravity             := Formula Planet Gravity\n')

  if pdata['class'] != 'Terr':
    out_file.write('Planet Temperature         := Formula Planet ' + pdata['class'] + ' Temperature\n')
  else:
    out_file.write('Planet Temperature         := Formula Planet Temperature\n')

  if pdata['atm'] == 'None':
    out_file.write('Planet Radiation           := Formula Planet None Radiation\n')
  else:
    out_file.write('Planet Radiation           := Formula Planet Radiation\n')

  out_file.write('Planet Surface Empty       := PlanetSurface_' + pdata['type'] + '_' + pdata['atm'].split()[0] + '_Empty.bmp\n')
  out_file.write('Planet Surface Colonized   := PlanetSurface_' + pdata['type'] + '_' + pdata['atm'].split()[0] + '_Colonized.bmp\n')
  out_file.write('\n')
# end write_planet()

gas_scale_size = {'Tiny': '5', 'Small': '6', 'Medium': '7', 'Large': '8', 'Huge': '9'}
oth_scale_size = {'Tiny': '2', 'Small': '3', 'Medium': '4', 'Large': '5', 'Huge': '6'}

def write_xfile(out_file, pdata, pnum):
  out_file.write('Name                                                 := Planet ' + pdata['class'] + ' ' + pdata['atm'].split()[0] + ' ' + pdata['size'] + ' ' + str(pnum) + '\n')
  out_file.write('XFile                                                := Planet1000.x\n')
  out_file.write('Override Texture Filename                            := Lacota\\Planet_' + pdata['class'] + '_' + pdata['atm'].split()[0] + '_' + str(pnum) + '.bmp\n')
  out_file.write('Base Model Radius                                    := 2.0\n')
  out_file.write('Starting Rotate X                                    := 50\n')
  out_file.write('Starting Rotate Y                                    := 0\n')
  out_file.write('Starting Rotate Z                                    := 25\n')

  if pdata['type'] in ['Gas']:
    scale_size = gas_scale_size[pdata['size']]
    glow_size = str(int(scale_size) * 7)
  elif pdata['type'] in ['Ringed', 'Ven Ringed', 'Ice Ringed', 'Terr Ringed']:
    scale_size = gas_scale_size[pdata['size']]
    glow_size = str((int(scale_size) - 1) * 7)
  else:
    scale_size = oth_scale_size[pdata['size']]
    glow_size = str(int(scale_size) * 7)


  out_file.write('Starting Scale X                                     := ' + str(scale_size) + '\n')
  out_file.write('Starting Scale Y                                     := ' + str(scale_size) + '\n')
  out_file.write('Starting Scale Z                                     := ' + str(scale_size) + '\n')
  out_file.write('Starting Position Offset X                           := 0\n')
  out_file.write('Starting Position Offset Y                           := 0\n')
  out_file.write('Starting Position Offset Z                           := 0\n')
  out_file.write('Continual Rotation Vector X                          := 0\n')
  out_file.write('Continual Rotation Vector Y                          := 0\n')
  out_file.write('Continual Rotation Vector Z                          := 25\n')
  out_file.write('Continual Rotation Speed                             := 0.0001\n')

  if pdata['atm'] != 'None':
    out_file.write('Brightness Offset Percent RGB                        := 100, 100, 100\n')
    out_file.write('Number of Bitmap Effects                             := 1\n')
    out_file.write('Effect 1 Name                                        := Planet Glow\n')
    out_file.write('Effect 1 Size                                        := ' + str(glow_size) + '\n')
    out_file.write('Effect 1 Duplicate Count                             := 2\n')
    out_file.write('Effect 1 Frame Change Rate                           := 0\n')
    out_file.write('Effect 1 Offset Position X                           := 0\n')
    out_file.write('Effect 1 Offset Position Y                           := 0\n')
    out_file.write('Effect 1 Offset Position Z                           := 0\n')
    out_file.write('Effect 1 Rotate X                                    := 0\n')
    out_file.write('Effect 1 Rotate Y                                    := 0\n')
    out_file.write('Effect 1 Rotate Z                                    := 0\n')
    out_file.write('Effect 1 Rotate Center Offset X                      := 0\n')
    out_file.write('Effect 1 Rotate Center Offset Y                      := 0\n')
    out_file.write('Effect 1 Rotate Center Offset Z                      := 0\n')
    out_file.write('Effect 1 Use Lighting                                := TRUE\n')
    out_file.write('Effect 1 Blend Source                                := Blend_One\n')
    out_file.write('Effect 1 Blend Dest                                  := Blend_One\n')
    out_file.write('Effect 1 Brightness Change Rate                      := 0\n')
    out_file.write('Effect 1 Random Brightness Start                     := FALSE\n')
    out_file.write('Effect 1 Starting Brightness RGB                     := ' + x_atm_glow[pdata['atm']] + '\n')
    out_file.write('Effect 1 Ending Brightness RGB                       := ' + x_atm_glow[pdata['atm']] + '\n')
    out_file.write('Effect 1 Always Behind Model                         := TRUE\n')

  out_file.write('\n')
# end write_xfile()

def write_star(out_file, sdata, snum):
  out_file.write('Stellar Object Name        := Star ' + sdata['color'] + ' ' + sdata['size'] + ' ' + str(snum) + '\n')
  out_file.write('Physical Type              := Star\n')
  out_file.write('XFile Class Name           := Star ' + sdata['color'] + ' ' + sdata['size'] + '\n')
  out_file.write('Maximum Model Size         := 40.0\n')
  out_file.write('Hex Size                   := ' + str(sdata['hex_size']) + '\n')
  out_file.write('Blocked Hex Radius         := ' + str(sdata['hex_size']) + '\n')
  out_file.write('Picture Index              := ' + str(sdata['pic'] + s_icon_offset[sdata['size']]) + '\n')
  out_file.write('Portrait Filename          := FQM\\star_' + (sdata['color'].replace(' ', '_')).lower() + '.bmp\n')
  out_file.write('Large Portrait Filename    := FQM\\star_' + (sdata['color'].replace(' ', '_')).lower() + '_lp.jpg\n')

  if sdata['color'] == 'Red Dim':
    out_file.write('Description                := ' + sdata['size'] + ', dim red star of moderate age.\n')
  else:
    out_file.write('Description                := ' + sdata['size'] + ', ' + sdata['color'].lower() + ' star of moderate age.\n')

  out_file.write('Star Size                  := ' + sdata['size'] + '\n')
  out_file.write('Star Age                   := ' + sdata['age'] + '\n')

  if sdata['color'] == 'Red Orange':
    out_file.write('Star Color                 := Orange\n')
  else:
    out_file.write('Star Color                 := ' + sdata['color'].split()[0] + '\n')

  out_file.write('Star Luminosity            := ' + sdata['lumin'] + '\n')
  out_file.write('\n')
#end write_star()

texture_num = 2

write_copy_file('stellar_top', p_file)
write_copy_file('xfile_top', x_file)


p_file.write('//------------------------------------------------------------------------------\n')
p_file.write('// Normal Planets\n')
p_file.write('//------------------------------------------------------------------------------\n')

x_file.write('//------------------------------------------------------------------------------\n')
x_file.write('// Normal Planets\n')
x_file.write('//------------------------------------------------------------------------------\n')

for t_atm in p_atm:
  for t_type in p_types:
    for t_class in p_classes[t_type]:
      for t_size in p_sizes:
        for i in range(1, texture_num + 1):
          pdata = {'atm': t_atm,
                   'type': t_type,
                   'class': t_class,
                   'size': t_size}

          write_planet(p_file, pdata, pad_num(i))

          write_xfile(x_file, pdata, pad_num(i))

s_sizes = ['Small', 'Medium', 'Large']

s_icon_offset = {'Small':  2,
                 'Medium': 1,
                 'Large':  0}

s_stat = [[4, 'Red Dim', 2, 'Average', 'Dim', 679],
          [4, 'Red', 2, 'Average', 'Dim', 679],
          [3, 'Red Orange', 2, 'Average', 'Dim', 676],
          [3, 'Orange', 2, 'Average', 'Dim', 676],
          [1, 'Green', 2, 'Average', 'Average', 685],
          [2, 'Yellow', 2, 'Average', 'Average', 685],
          [1, 'White', 3, 'Average', 'Bright', 682],
          [1, 'Blue', 3, 'Average', 'Bright', 673]]

p_file.write('//------------------------------------------------------------------------------\n')
p_file.write('// Stars\n')
p_file.write('//------------------------------------------------------------------------------\n')
p_file.write('//\n')
p_file.write('// Proportions for appearance:\n')
p_file.write('// dim red      4\n')
p_file.write('// red          4\n')
p_file.write('// red-orange   3\n')
p_file.write('// orange       3\n')
p_file.write('// green        1\n')
p_file.write('// yellow       2\n')
p_file.write('// white        1\n')
p_file.write('// blue         1\n')
p_file.write('//\n')

for s in s_stat:
  for i in range(1, s[0] + 1):
    for s_size in s_sizes:
      sdata = {'color': s[1],
               'hex_size': s[2],
               'size': s_size,
               'age': s[3],
               'lumin': s[4],
               'pic': s[5]}

      write_star(p_file, sdata, pad_num(i))

write_copy_file('stellar_bottom', p_file)
write_copy_file('xfile_bottom', x_file)

x_file.close()
p_file.close()
